# -*- coding: utf-8 -*-

dictionary = \
{
    'lang':                             u'hu',
#admin fields
    'admin':                            u'Adminisztrátor',
    'catalogs':                         u'Katalógusok',
    'journals':                         u'Naplók',
    'tables':                           u'Táblák',
    'reports':                          u'Riportok',
    'details':                          u'Részletek',
    'id':                               u'Rekord Azon',
    'deleted_flag':                     u'Töröltve',
    'caption':                          u'Fejléc',
    'name':                             u'Név',
    'table_field_name':                 u'Field name',
    'table_name':                       u'Tábla',
    'template':                         u'Riport sablon',
    'params_template':                  u'Paraméter sablon',
    'view_template':                    u'Nézet sablon',
    'visible':                          u'Látható',
    'client_module':                    u'Kliens modul',
    'server_module':                    u'Szerver modul',
    'project':                          u'Projekt',
    'users':                            u'Felhasználók',
    'roles':                            u'Szerepek',
    'privileges':                       u'Jogok',
    'tasks':                            u'Task',
    'safe_mode':                        u'Safe mód',
    'language':                         u'Nyelv',
    'author':                           u'Szerző',
    'interface':                        u'Interface',
    'db_type':                          u'DB tipus',
    'db_name':                          u'File név',
    'alias':                            u'Adatbázis',
    'data_type':                        u'Tipus',
    'filter_type':                      u'Szűrő tipus',
    'size':                             u'Máret',
    'object':                           u'Lookup tábla',
    'object_field':                     u'Lookup oszlop',
    'lookup_values':                    u'Lookup érték',
    'master_field':                     u'Mester oszlop',
    'required':                         u'Kötelező',
    'calculated':                       u'Számolt.',
    'default':                          u'Alap',
    'read_only':                        u'Csak olvasható',
    'alignment':                        u'Igazítás',
    'active':                           u'Aktív',
    'date':                             u'Dátum',
    'role':                             u'Szerep',
    'info':                             u'Információ',
    'item':                             u'Item',
    'can_view':                         u'Nézhet',
    'can_create':                       u'Létrehozhat',
    'can_edit':                         u'Szerkeszthet',
    'can_delete':                       u'Törölhet',
    'fields':                           u'Mezők',
    'field':                            u'Mező',
    'filter':                           u'Szűrő',
    'filters':                          u'Szűrők',
    'index':                            u'Index',
    'index_name':                       u'Index név',
    'report_params':                    u'Riport paraméter',
    'error':                            u'Hiba',
#admin interface
    'db':                               u'Adatbázis',
    'export':                           u'Export',
    'import':                           u'Import',
    'viewing':                          u'Megnéz',
    'editing':                          u'Szerkeszt',
    'filters':                          u'Szűrők',
    'order':                            u'Sorrend',
    'indices':                          u'Indexek',
    'foreign_keys':                     u'Foreign keys',
    'select_all':                       u'Kijelöl mind',
    'unselect_all':                     u'Semmit jelöl',
    'project_params':                   u'Paraméterek',
    'project_locale':                   u'Területi',
    'reserved_word':                    u'A nev foglalt kulcsszó',
    'is_edited':                        u'%s is already open for editing. Nevertheless, open it?',
    'is_edited_by':                     u'%s is already open for editing by %s. Nevertheless, open it?',
#editor
    'case_sensitive':                   u'Betü érzékeny',
    'whole_words':                      u'Teljesz szóra keres',
    'in_task':                          u'a taskban',
    'text_not_found':                   u'Nem található.\nBontsa meg és újra?',
    'text_changed':                     u'A modul megváltozott.\nBezárás elött legyen mentés?',
    'go_to_line':                       u'Ugrás a sorhoz',
    'go_to':                            u'Ugrás',
    'line':                             u'sor',
#admin editors
    'caption_name':                     u'Név',
    'caption_word_wrap':                u'Tördelás',
    'caption_expand':                   u'Bővit',
    'caption_edit':                     u'Szerkeszt',
    'caption_descening':                u'Csök.',
#admin messages
    'fill_task_attrubutes':             u'Töltse ki a címet és az adat bázis tipusát.',
    'can_not_connect':                  u"%s adat bázishoz nem lehet csatlakozni.",
    'field_used_in_filters':            u"%s mező nem törölhető.\n A szűrő definoció használja:\n%s",
    'field_used_in_fields':             u"%s mező nem törölhető.\n A mező definoció használja:\n%s",
    'field_used_in_indices':            u"%s mező nem törölhető.\n Az index definoció használja:\n%s",
    'field_is_system':                  u"Rendszer mező nem törölhető.",
    'detail_mess':                      u'%s - részlete %s',
    'item_used_in_items':               u"%s tétel nem törölhető.\n A tétel definició használja:\n%s",
    'field_mess':                       u'%s - mező %s',
    'item_used_in_fields':              u"s tétel nem törölhető.\n A mező definició használja:\n%s",
    'param_mess':                       u'%s - paraméter %s',
    'item_used_in_params':              u"s tétel nem törölhető.\n A paraméter definició használja:\n%s",
    'invalid_name':                     u'Érvénytelen név.',
    'invalid_field_name':               u'Érvénytelen mezőnév.',
    'type_is_required':                 u'A mező tipus kötelező.',
    'index_name_required':              u'Az index név kötelező.',
    'index_fields_required':            u'Az index mező kötelező.',
    'cant_delete_group':                u"Csoport nem törölhető.",
    'object_field_required':            u'Lookup mező szükséges',
    'no_tasks_ptoject':                 u'A project nem tartalmaz taskot.',
    'stop_server':                      u'Szerver álj',
#interface buttons and labels
    'yes':                              u'Igen',
    'no':                               u'Nem',
    'ok':                               u'OK',
    'cancel':                           u'Mégsem',
    'delete':                           u'Töröl',
    'new':                              u'Új',
    'edit':                             u'Szerkeszt',
    'copy':                             u'Másol',
    'print':                            u'Print',
    'save':                             u'Ment',
    'open':                             u'Megnyit',
    'close':                            u'Bezár',
    'select':                           u'Kiválaszt',
    'filter':                           u'Szűrő',
    'apply':                            u'Alkalmaz',
    'find':                             u'Keres',
    'replace':                          u'Cserél',
    'view':                             u'megnéz',
    'log_in':                           u'Belép',
    'login':                            u'Belépés',
    'password':                         u'Jelszó',
    'log_out':                          u'Kilép',
#runtime messages
    'invalid_int':                      u'%s rossz érték - egésznek kell lenni',
    'invalid_float':                    u'%s rossz érték - lebegőpontosnak kell lenni',
    'invalid_cur':                      u'%s rossz érték - pénznek kell lenni',
    'invalid_date':                     u'%s rossz érték - dátumnak kell lenni',
    'invalid_bool':                     u'%s rossz érték - logikainak kell lenni',
    'invalid_value':                    u'%s rossz érték',
    'value_required':                   u'Nem lehet üres',
    'invalid_length':                   u'A szöveg tul hosszú, maximum hossz - %s',
    'save_changes':                     u'Módosítás történt. Legyen mentés?',
    'delete_record':                    u'Töröljük a rekordot?',
    'no_record':                        u'Record is not selected.',
    'server_request_error':             u'Szerver kérelmi hiba',
    'cant_delete_used_record':          u"A rekord nem törölhető használatban van.",
    'website_maintenance':              u"A szerver ideglenesen nem elérhető.",
    'server_restarted':                 u"Server restarted. Press F5 to reload the page.",
    'version_changed':                  u"Verzió váltás F5 a frissítésekhez",
    'no_task':                          u'The project is loading or the creation of the project has not been completed yet.',
    'items_selected':                   u"Kiválasztva %s tétel",
    'invalid_range':                    u'Index hiba',
    'range_from':                       u'tól',
    'range_to':                         u'ig',
#rights messages
    'cant_view':                        u'%s: nincs jogosultjága megnézni.',
    'cant_create':                      u'%s: nincs jogosultjága létrehozni.',
    'cant_edit':                        u'%s: nincs jogosultjága szerkeszteni.',
    'cant_delete':                      u'%s: nincs jogosultjága törölni.',
#calendar
    'week_start':                        0,
    'days_min':                         [u'V', u'H', u'Ke', u'Sze', u'Cs', u'P', u'Szo', u'V'],
    'months':                           [u'Január', u'Február', u'Március', u'Április', u'Május', u'Junius', u'Julius', u'Augusztus', u'Szeptember', u'Oktober', u'November', u'December'],
    'months_short':                     [u'Jan', u'Feb', u'Már', u'Ápr', u'Máj', u'Jun', u'Jul', u'Aug', u'Sep', u'Okt', u'Nov', u'Dec'],
#grid
    'page':                             u'Lap',
    'of':                               u'a',
#history
    'created':                        u'Created',
    'modified':                       u'Modified',
    'deleted':                        u'Deleted',
    'by_user':                        u'by',
    'old_value':                      u'old value',
    'new_value':                      u'new value'

}
