# -*- coding: utf-8 -*-

dictionary = \
{
    'lang':                             u'pt',
#admin fields
    'admin':                            u'Administrador',
    'catalogs':                         u'Catalogos',
    'journals':                         u'Transações',
    'tables':                           u'Tabelas',
    'reports':                          u'Relatórios',
    'details':                          u'Detalhes',
    'id':                               u'ID de Registro',
    'deleted_flag':                     u'Excluído?',
    'caption':                          u'Título',
    'name':                             u'Nome',
    'table_field_name':                 u'Field name',
    'table_name':                       u'Nome da Tabela',
    'template':                         u'Relatório (modelo)',
    'params_template':                  u'Parâmetros',
    'view_template':                    u'Relatório (modelo)',
    'visible':                          u'Visível',
    'client_module':                    u'Cliente (módulo)',
    'server_module':                    u'Servidor (módulo)',
    'project':                          u'Projeto',
    'users':                            u'Usuários',
    'roles':                            u'Perfis',
    'privileges':                       u'Privilégios',
    'tasks':                            u'Tarefas',
    'safe_mode':                        u'Modo Seguro',
    'language':                         u'Idioma',
    'author':                           u'Autor',
    'interface':                        u'Interface',
    'db_type':                          u'BD Tipo',
    'db_name':                          u'Nome do Arquivo',
    'alias':                            u'Banco de Dados',
    'data_type':                        u'Tipo',
    'filter_type':                      u'Tipo de Filtro',
    'size':                             u'Tamanho',
    'object':                           u'Item da Pesquisa',
    'object_field':                     u'Campo da Pesquisa',
    'lookup_values':                    u'Valor da Pesquisa',
    'master_field':                     u'Campo Principal',
    'required':                         u'Obrigatório',
    'calculated':                       u'Calculado',
    'default':                          u'Padrão',
    'read_only':                        u'Somente Leitura',
    'alignment':                        u'Alinhamento',
    'active':                           u'Ativo',
    'date':                             u'Data',
    'role':                             u'Perfil',
    'info':                             u'Informação',
    'item':                             u'Item',
    'can_view':                         u'Pode ver',
    'can_create':                       u'Pode criar',
    'can_edit':                         u'Pode editar',
    'can_delete':                       u'Pode excluir',
    'fields':                           u'Campos',
    'field':                            u'Campo',
    'filter':                           u'Filtro',
    'filters':                          u'Filtros',
    'index':                            u'Índice',
    'index_name':                       u'Nome do Índice',
    'report_params':                    u'Parâmetros do Relatório',
    'error':                            u'Erro',
#admin interface
    'db':                               u'Banco de Dados',
    'export':                           u'Exportar',
    'import':                           u'Importar',
    'viewing':                          u'Ver',
    'editing':                          u'Editar',
    'filters':                          u'Filtros',
    'order':                            u'Ordem',
    'indices':                          u'Índices',
    'foreign_keys':                     u'Change Extrangeira',
    'select_all':                       u'Selecionar tudo',
    'unselect_all':                     u'Deselecionar tudo',
    'project_params':                   u'Parâmetros',
    'project_locale':                   u'Localidade',
    'reserved_word':                    u'O nome é uma palavra reservada',
    'is_edited':                        u'%s is already open for editing. Nevertheless, open it?',
    'is_edited_by':                     u'%s is already open for editing by %s. Nevertheless, open it?',
#editor
    'case_sensitive':                   u'Diferenciar Maiúsculas e Minúsculas',
    'whole_words':                      u'Concidir palavras inteiras',
    'in_task':                          u'Na Tarefa',
    'text_not_found':                   u'Texto não encontrado.\nModificar a busca e tentar novamente?',
    'text_changed':                     u'O Módulo foi modificado.\nSalvar antes de fechar?',
    'go_to_line':                       u'Ir para linha',
    'go_to':                            u'Ir para',
    'line':                             u'Linha',
#admin editors
    'caption_name':                     u'Nome',
    'caption_word_wrap':                u'Cortar',
    'caption_expand':                   u'Expandir',
    'caption_edit':                     u'Editar',
    'caption_descening':                u'Descendente',
#admin messages
    'fill_task_attrubutes':             u'Preencher nos atributos de título, nome e banco de dados.',
    'can_not_connect':                  u"Não foi possível conectar ao banco de dados. %s",
    'field_used_in_filters':            u"Não foi possível deletar o campo %s.\n Está sendo utilizado nas definições de filtro:\n%s",
    'field_used_in_fields':             u"Não foi possível deletar o campo %s.\n Está sendo utilizado nas definições de campo:\n%s",
    'field_used_in_indices':            u"Não foi possível deletar o campo %s.\n Está sendo utilizado nas definições de índice:\n%s",
    'field_is_system':                  u"Não é possível deletar um campo de padrão.",
    'detail_mess':                      u'%s - detalhe %s',
    'item_used_in_items':               u"Nao foi possível deletar o item %s.\n Está sendo utilizado nas definições de item:\n%s",
    'field_mess':                       u'%s - campo %s',
    'item_used_in_fields':              u"Não foi possível deletar o item %s.\n Está sendo utilizado nas definições de campo:\n%s",
    'param_mess':                       u'%s - parâmetros %s',
    'item_used_in_params':              u"Não foi possível deletar o item %s.\n Está sendo utilizado nas definições de parâmetros:\n%s",
    'invalid_name':                     u'o Nome é inválido.',
    'invalid_field_name':               u'O nome do campo é inválido.',
    'type_is_required':                 u'O tipo do campo é requerido.',
    'index_name_required':              u'O índice é requerido.',
    'index_fields_required':            u'Não há campos de índice requeridos.',
    'cant_delete_group':                u"Não é possível deletar um grupo.",
    'object_field_required':            u'Um campo de pesquisa é obrigatório.',
    'no_tasks_ptoject':                 u'Não existem tarefas no projeto.',
    'stop_server':                      u'Parar servidor',
#interface buttons and labels
    'yes':                              u'Sim',
    'no':                               u'Não',
    'ok':                               u'OK',
    'cancel':                           u'Cancelar',
    'delete':                           u'Excluir',
    'new':                              u'Novo',
    'edit':                             u'Editar',
    'copy':                             u'Copiar',
    'print':                            u'Imprimir',
    'save':                             u'Guardar',
    'open':                             u'Abrir',
    'close':                            u'Fechar',
    'select':                           u'Selecionar',
    'filter':                           u'Filtrar',
    'apply':                            u'Aplicar',
    'find':                             u'Pesquisar',
    'replace':                          u'Substituir',
    'view':                             u'Ver',
    'log_in':                           u'Logar',
    'login':                            u'Usuário',
    'password':                         u'Senha',
    'log_out':                          u'Sair',
#runtime messages
    'invalid_int':                      u'%s valor inválido - deve ser um inteiro',
    'invalid_float':                    u'%s valor inválido - deve ser um decimal',
    'invalid_cur':                      u'%s valor inválido - deve ser uma moeda',
    'invalid_date':                     u'%s valor inválido - deve ser uma data',
    'invalid_bool':                     u'%s valor inválido - deve ser um boleano',
    'invalid_value':                    u'%s valor inválido',
    'value_required':                   u'um valor é requerido',
    'invalid_length':                   u'O texto ultrapassa o tamanho máximo permitido - %s',
    'save_changes':                     u'Os dados foram alterados. Você gostaria de salvar as modificações?',
    'delete_record':                    u'Deletar o registro?',
    'no_record':                        u'Record is not selected.',
    'server_request_error':             u'Erro de requisição ao servidor',
    'cant_delete_used_record':          u"Impossível deletar o registro. Ele está sendo usado.",
    'website_maintenance':              u"O servidor está temporáriamente indisponível.",
    'server_restarted':                 u"Server restarted. Press F5 to reload the page.",
    'version_changed':                  u"Versão modificada. Pressione F5 para atualizar",
    'no_task':                          u'The project is loading or the creation of the project has not been completed yet.',
    'items_selected':                   u"selecionado: %s",
    'invalid_range':                    u'extensão invalida',
    'range_from':                       u'de',
    'range_to':                         u'para',
#rights messages
    'cant_view':                        u'%s: você não tem permissões para ver',
    'cant_create':                      u'%s: você não tem permissões para criar',
    'cant_edit':                        u'%s: você não tem permissões para editar',
    'cant_delete':                      u'%s: você não tem permissões para excluir',
#calendar
    'week_start':                        0,
    'days_min':                         [u'Dom', u'Seg', u'Ter', u'Qua', u'Qui', u'Sex', u'Sáb', u'Dom'],
    'months':                           [u'Janeiro', u'Fevereiro', u'Março', u'Abril', u'Maio', u'Junho', u'Julho', u'Agosto', u'Setembro', u'Outubro', u'Novembro', u'Dezembro'],
    'months_short':                     [u'Jan', u'Fev', u'Mar', u'Abr', u'Mai', u'Jun', u'Jul', u'Ago', u'Set', u'Out', u'Nov', u'Dez'],
#grid
    'page':                             u'Página',
    'of':                               u'de',
#history
    'created':                        u'Created',
    'modified':                       u'Modified',
    'deleted':                        u'Deleted',
    'by_user':                        u'by',
    'old_value':                      u'old value',
    'new_value':                      u'new value'
}
