body {
    padding-top: 60px;
    overflow-x: hidden;
}

.title {
    background-color: #18bc9c;
    color: #fff;
    margin: 0px;
    padding: 12px;
}

#doc-body {
    padding-left: 60px;
    padding-right: 40px;
}

#doc-body img {
    display: block;
    max-width: 100%;
    max-height: 100%;
    margin-top: 10px;
    margin-bottom: 10px;
    margin-left: auto;
    margin-right: auto;
}

#dev-warning {
    margin-top: 4px;
}

a {
    color: #107562;
/*
    color: #15A388;
*/
}

a:hover, a:focus {
    color: #18bc9c;
    text-decoration: underline;
}

#doc-sidebar {
    padding-right: 40px;
}

footer {
    padding: 20px 0;
    background-color: #233140;
    color: #fff;
}

a.headerlink {
    color: #ffffff;
}
a.headerlink:hover {
    color: #18bc9c;;
}

.highlight {
    background-color: #cbf5ed;
}

.note, .admonition {
    border: 1px solid #18bc9c;
    padding: 10px;
    margin-bottom: 4px;
}

.admonition-title {
    color: #107562;
}

.sphinxsidebarwrapper h3 {
    padding-bottom: 16px;
    border-bottom: 1px solid #dddddd;
}

.table-bordered  {
    border-left: 1px solid #dddddd;
}

.table-bordered th:first-child,
.table-bordered td:first-child {
    border-left: 0;
}
table caption {
    text-align: center;
    color: #2c3e50;
}

th {
    text-align: center;
}

table {
    margin-bottom: 20px;
}

.docutils.literal {
    color: #076654;
    font-weight: bold;
    font-size: 14px;
    background-color: #ffffff;
}
